﻿using System;
using System.Collections.Generic;
using TSLab.Script.Helpers;

// ReSharper disable UnusedAutoPropertyAccessor.Global
// ReSharper disable MemberCanBePrivate.Global
// ReSharper disable UnusedMember.Global
namespace TSLab.Script.Handlers
{
    public abstract class BasePeriodIndicatorHandler
    {
        [HandlerParameter(true, "20", Min = "10", Max = "100", Step = "5")]
        public int Period
        {
            get;
            set;
        }
    }

    [HandlerCategory("Indicators")]
    public class RSI : BasePeriodIndicatorHandler, IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            return Series.RSI(source, Period);
        }
    }

    [HandlerCategory("Indicators")]
    public class SummFor : BasePeriodIndicatorHandler, IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            return Series.SummFor(source, Period);
        }
    }

    [HandlerCategory("Indicators")]
    public class SMA : BasePeriodIndicatorHandler, IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            return Series.SMA(source, Period);
        }
    }

    [HandlerCategory("Indicators")]
    public class StDev : BasePeriodIndicatorHandler, IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            return Series.StDev(source, Period);
        }
    }

    [HandlerCategory("Indicators")]
    public class EMA : BasePeriodIndicatorHandler, IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            return Series.EMA(source, Period);
        }
    }

    [HandlerName("TR")]
    [HandlerCategory("Indicators")]
    public class TrueRange : IBar2DoubleHandler
    {
        public IList<double> Execute(ISecurity source)
        {
            return Series.TrueRange(source.Bars);
        }
    }

    [HandlerName("ATR")]
    [HandlerCategory("Indicators")]
    public class AverageTrueRange : BasePeriodIndicatorHandler, IBar2DoubleHandler
    {
        public IList<double> Execute(ISecurity source)
        {
            return Series.AverageTrueRange(source.Bars, Period);
        }
    }

    [HandlerCategory("Indicators")]
    public class CCI : BasePeriodIndicatorHandler, IBar2DoubleHandler
    {
        public IList<double> Execute(ISecurity source)
        {
            return Series.CCI(source.Bars, Period);
        }
    }

    [HandlerName("Typical Price")]
    [HandlerCategory("Indicators")]
    public class TypicalPrice : IBar2DoubleHandler
    {
        public IList<double> Execute(ISecurity source)
        {
            return Series.TypicalPrice(source.Bars);
        }
    }

    [HandlerName("Bollinger Bands (+)")]
    [HandlerCategory("Indicators")]
    public class BollingerBands1 : BasePeriodIndicatorHandler, IDouble2DoubleHandler
    {
        [HandlerParameter(true, "2", Min = "0.5", Max = "3", Step = "0.5")]
        public double Coef
        {
            get;
            set;
        }

        public IList<double> Execute(IList<double> source)
        {
            return Series.BollingerBands(source, Period, Coef, true);
        }
    }

    [HandlerName("Bollinger Bands (-)")]
    [HandlerCategory("Indicators")]
    public class BollingerBands2 : BasePeriodIndicatorHandler, IDouble2DoubleHandler
    {
        [HandlerParameter(true, "2", Min = "0.5", Max = "3", Step = "0.5")]
        public double Coef
        {
            get;
            set;
        }

        public IList<double> Execute(IList<double> source)
        {
            return Series.BollingerBands(source, Period, Coef, false);
        }
    }

    [HandlerDecimals(2)]
    [HandlerCategory("Indicators")]
    public class Relative : IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            double v = source.Count == 0 ? 0 : source[0];
            if (v == 0)
                v = double.MaxValue;
            IList<double> list = new List<double>(source.Count);
            for (int i = 0; i < source.Count; i++)
                list.Add((source[i] - v) / v * 100);
            return list;
        }
    }

    [HandlerCategory("Indicators")]
    public class Highest : BasePeriodIndicatorHandler, IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            return Series.Highest(source, Period);
        }
    }

    [HandlerCategory("Indicators")]
    public class Lowest : BasePeriodIndicatorHandler, IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            return Series.Lowest(source, Period);
        }
    }

    [HandlerCategory("TradeMath")]
    public class Multiply : IDouble2DoubleHandler
    {
        [HandlerParameter(true, "2", Min = "0.5", Max = "5", Step = "0.5")]
        public double Coef
        {
            get;
            set;
        }

        public IList<double> Execute(IList<double> bars)
        {
            IList<double> list = new List<double>(bars.Count);
            for (int i = 0; i < bars.Count; i++)
                list.Add(bars[i] * Coef);
            return list;
        }
    }
}
// ReSharper restore MemberCanBePrivate.Global
// ReSharper restore UnusedAutoPropertyAccessor.Global
// ReSharper restore UnusedMember.Global
