﻿using System.Collections.Generic;
using TSLab.Script.Helpers;

// ReSharper disable UnusedAutoPropertyAccessor.Global
// ReSharper disable MemberCanBePrivate.Global
// ReSharper disable UnusedMember.Global
namespace TSLab.Script.Handlers
{
    public class MACDBase : IContextUses
    {
        protected IList<double> CalcMACD(IList<double> source, int p1, int p2)
        {
            var ema1 = Series.EMA(source, p1);
            var ema2 = Series.EMA(source, p2);
            var res = new List<double>(ema1.Count);
            for (int i = 0; i < ema1.Count; i++)
            {
                res.Add(ema1[i] - ema2[i]);
            }
            return res;
        }

        public IContext Context { get; set; }
    }

    [HandlerCategory("Indicators")]
    public class MACD : MACDBase, IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            return CalcMACD(source, 12, 26);
        }
    }

    [HandlerName("MACD Ext")]
    [HandlerCategory("Indicators")]
    public class MACDEx : MACDBase, IDouble2DoubleHandler
    {
        [HandlerParameter(true, "26", Min = "10", Max = "40", Step = "1")]
        public int Period2 { get; set; }

        [HandlerParameter(true, "12", Min = "5", Max = "40", Step = "1")]
        public int Period1 { get; set; }

        public IList<double> Execute(IList<double> source)
        {
            return CalcMACD(source, Period1, Period2);
        }
    }

    [HandlerName("MACD Signal")]
    [HandlerCategory("Indicators")]
    public class MACDSig : IDouble2DoubleHandler
    {
        [HandlerParameter(true, "9", Min = "3", Max = "20", Step = "1")]
        public int Period
        {
            get;
            set;
        }

        public IList<double> Execute(IList<double> source)
        {
            return Series.EMA(source, Period);
        }
    }
}
// ReSharper restore MemberCanBePrivate.Global
// ReSharper restore UnusedAutoPropertyAccessor.Global
// ReSharper restore UnusedMember.Global
