﻿using System;
using System.Collections.Generic;
using TSLab.Script.Helpers;

// ReSharper disable UnusedAutoPropertyAccessor.Global
// ReSharper disable MemberCanBePrivate.Global
// ReSharper disable UnusedMember.Global
namespace TSLab.Script.Handlers
{
    [HandlerName("Momentum %")]
    [HandlerCategory("Indicators")]
    public class MomentumPct : BasePeriodIndicatorHandler, IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            IList<double> list = new double[source.Count];
            for (int i = 0; i < source.Count; i++)
            {
                int k = Math.Max(0, i - Period);
                list[i] = (source[i] / Math.Max(1e-5, source[k]) * 100);
            }
            return list;
        }
    }

    [HandlerCategory("Indicators")]
    public class Momentum : BasePeriodIndicatorHandler, IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            IList<double> list = new double[source.Count];
            for (int i = 0; i < source.Count; i++)
            {
                int k = Math.Max(0, i - Period);
                list[i] = source[i] - source[k];
            }
            return list;
        }
    }

    [HandlerName("Chande Momentum Oscillator")]
    [HandlerCategory("Indicators")]
    public class MomentumOsc : BasePeriodIndicatorHandler, IDouble2DoubleHandler
    {
        public IList<double> Execute(IList<double> source)
        {
            IList<double> list = new double[source.Count];
            IList<double> m1 = new double[source.Count];
            IList<double> m2 = new double[source.Count];
            double osc1 = 0;
            double osc2 = 0;
            for (int i = 1; i < source.Count; i++)
            {
                var m = source[i] - source[i - 1];
                m1[i] = m > 0 ? m : 0;
                m2[i] = m < 0 ? -m : 0;
                osc1 += m1[i];
                osc2 += m2[i];
                if (i >= Period)
                {
                    osc1 -= m1[i - Period];
                    osc2 -= m2[i - Period];
                }
                list[i] = (osc1 - osc2)/(osc1 + osc2)*100;
            }
            return list;
        }
    }

}
// ReSharper restore MemberCanBePrivate.Global
// ReSharper restore UnusedAutoPropertyAccessor.Global
// ReSharper restore UnusedMember.Global
