﻿using System.Collections.Generic;

namespace TSLab.Script.Handlers
{
// ReSharper disable MemberCanBePrivate.Global
    [HandlerName("ParabolicSAR")]
    [HandlerCategory("Indicators")]
    public class ParabolicSAR : IBar2DoubleHandler
    {
        [HandlerParameter(true, "0.02", Min = "0.01", Max = "0.1", Step = "0.01", Name = "Acceleration Start")]
// ReSharper disable UnusedAutoPropertyAccessor.Global
        public double AccelerationStart { get; set; }

        [HandlerParameter(true, "0.02", Min = "0.01", Max = "0.1", Step = "0.01", Name = "Acceleration Step")]
        public double AccelerationStep { get; set; }

        [HandlerParameter(true, "0.2", Min = "0.1", Max = "0.4", Step = "0.1", Name = "Acceleration Max")]
        public double AccelerationMax { get; set; }
// ReSharper restore UnusedAutoPropertyAccessor.Global
// ReSharper restore MemberCanBePrivate.Global

//| Parabolic Sell And Reverse system                                |
        public IList<double> Execute(ISecurity source)
        {
            int count = source.Bars.Count;
            var sarBuffer = new double[count];
            var lowPrices = source.LowPrices;
            var highPrices = source.HighPrices;

            if(count > 1)
            {
                bool dirlong = highPrices[1] > highPrices[0] || lowPrices[1] > lowPrices[0];
                sarBuffer[0] = sarBuffer[1] = dirlong ? lowPrices[0] : highPrices[0];
                double start = AccelerationStart;
                double lastLow = lowPrices[1];
                double lastHigh = highPrices[1];
                double ep = dirlong ? lastHigh : lastLow;
                
                for (int i = 2; i < count; i++)
                {
                    var priceLow = lowPrices[i];
                    var priceHigh = highPrices[i];
                    //--- check for reverse from long to short
                    if (dirlong && priceLow < sarBuffer[i - 1])
                    {
                        start = AccelerationStart;
                        dirlong = false;
                        ep = priceLow;
                        lastLow = priceLow;
                        sarBuffer[i] = lastHigh;
                        continue;
                    }
                    //--- check for reverse from short to long  
                    if (!dirlong && priceHigh > sarBuffer[i - 1])
                    {
                        start = AccelerationStart;
                        dirlong = true;
                        ep = priceHigh;
                        lastHigh = priceHigh;
                        sarBuffer[i] = lastLow;
                        continue;
                    }
                    //sar(i) = sar(i-1)+start*(ep-sar(i-1))
                    var price = sarBuffer[i - 1];
                    var sar = price + start * (ep - price);
                    //----
                    if (dirlong)
                    {
                        if (ep < priceHigh && (start + AccelerationStep) <= AccelerationMax)
                            start += AccelerationStep;
                        //----
                        if (priceHigh < highPrices[i - 1] && i == 2)
                            sar = sarBuffer[i - 1];
                        price = lowPrices[i - 1];
                        //----
                        if (sar > price)
                            sar = price;
                        price = lowPrices[i - 2];
                        //----
                        if (sar > price)
                            sar = price;
                        //----
                        if (sar > priceLow)
                        {
                            start = AccelerationStart;
                            dirlong = false;
                            ep = priceLow;
                            lastLow = priceLow;
                            sarBuffer[i] = lastHigh;
                            continue;
                        }
                        //----
                        if (ep < priceHigh)
                        {
                            lastHigh = priceHigh;
                            ep = priceHigh;
                        }
                    }     //dir-long
                    else
                    {
                        if (ep > priceLow && (start + AccelerationStep) <= AccelerationMax)
                            start += AccelerationStep;
                        //----
                        if (priceLow < lowPrices[i - 1] && i == 2)
                            sar = sarBuffer[i - 1];
                        price = highPrices[i - 1];
                        //----
                        if (sar < price)
                            sar = price;
                        price = highPrices[i - 2];
                        //----
                        if (sar < price)
                            sar = price;
                        //----
                        if (sar < priceHigh)
                        {
                            start = AccelerationStart;
                            dirlong = true;
                            ep = priceHigh;
                            lastHigh = priceHigh;
                            sarBuffer[i] = lastLow;
                            continue;
                        }
                        //----
                        if (ep > priceLow)
                        {
                            lastLow = priceLow;
                            ep = priceLow;
                        }
                    }     //dir-short
                    sarBuffer[i] = sar;
                }
            }
            return sarBuffer;
        }
    }
}
